/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.cache;

import com.liferay.portal.kernel.cache.PortalCacheException;
import com.liferay.portal.kernel.cache.PortalCacheManagerListener;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * @author Tina Tian
 */
public class AggregatedPortalCacheManagerListener
	implements PortalCacheManagerListener {

	public boolean addPortalCacheListener(
		PortalCacheManagerListener portalCacheManagerListener) {

		if (portalCacheManagerListener == null) {
			return false;
		}

		return _portalCacheManagerListeners.add(portalCacheManagerListener);
	}

	public void clearAll() {
		_portalCacheManagerListeners.clear();
	}

	@Override
	public void dispose() throws PortalCacheException {
		for (PortalCacheManagerListener portalCacheManagerListener :
				_portalCacheManagerListeners) {

			portalCacheManagerListener.dispose();
		}
	}

	public Set<PortalCacheManagerListener> getPortalCacheManagerListeners() {
		return Collections.unmodifiableSet(_portalCacheManagerListeners);
	}

	@Override
	public void init() throws PortalCacheException {
		for (PortalCacheManagerListener portalCacheManagerListener :
				_portalCacheManagerListeners) {

			portalCacheManagerListener.init();
		}
	}

	@Override
	public void notifyPortalCacheAdded(String portalCacheName) {
		for (PortalCacheManagerListener portalCacheManagerListener :
				_portalCacheManagerListeners) {

			portalCacheManagerListener.notifyPortalCacheAdded(portalCacheName);
		}
	}

	@Override
	public void notifyPortalCacheRemoved(String portalCacheName) {
		for (PortalCacheManagerListener portalCacheManagerListener :
				_portalCacheManagerListeners) {

			portalCacheManagerListener.notifyPortalCacheRemoved(
				portalCacheName);
		}
	}

	public boolean removePortalCacheListener(
		PortalCacheManagerListener portalCacheManagerListener) {

		if (portalCacheManagerListener == null) {
			return false;
		}

		return _portalCacheManagerListeners.remove(portalCacheManagerListener);
	}

	private final Set<PortalCacheManagerListener> _portalCacheManagerListeners =
		new CopyOnWriteArraySet<>();

}