/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.cache;

import com.liferay.portal.kernel.model.MVCCModel;

import java.io.Serializable;

/**
 * @author Shuyang Zhou
 */
public class MVCCPortalCache<K extends Serializable, V extends MVCCModel>
	extends PortalCacheWrapper<K, V> {

	public MVCCPortalCache(LowLevelCache<K, V> lowLevelCache) {
		super(lowLevelCache);

		_lowLevelCache = lowLevelCache;
	}

	@Override
	public boolean isMVCC() {
		return true;
	}

	@Override
	public void put(K key, V value) {
		put(key, value, DEFAULT_TIME_TO_LIVE);
	}

	@Override
	public void put(K key, V value, int timeToLive) {
		while (true) {
			V oldValue = _lowLevelCache.get(key);

			if (oldValue == null) {
				oldValue = _lowLevelCache.putIfAbsent(key, value, timeToLive);

				if (oldValue == null) {
					return;
				}
			}

			if ((value.getMvccVersion() < oldValue.getMvccVersion()) ||
				_lowLevelCache.replace(key, oldValue, value, timeToLive)) {

				return;
			}
		}
	}

	private final LowLevelCache<K, V> _lowLevelCache;

}